/**
 * \defgroup filter 32bit filter algorithm implementation
 * \ingroup libbfin32
 * \details
 * A set of the FIR filter group. The FIR filter, the Interpolator and the Decimators
 * are implemented in  fr32_fir(), fr32_interpolator() and fr32_decimator(), respectively.
 *
 * All of these routines uses an TFIR32 variable, to keep the internal variable and
 * parameters. So, fr32_fir_init() routine have to be called at least and only once before
 * these filter routnes are invoked.
 */
/*@{*/

/**
 * \file fr32_fir.h
 * \date 2014/Mar/23
 * \author takemasa
 * \brief Basic FIR filter implementation
 */

#ifndef FR32_FIR_H_
#define FR32_FIR_H_

#include <fract.h>

/**
 * \brief FIR filter structure.
 */
typedef struct
{
	int taps;              ///< tap number of FIR filter. This have to be integer multiple of following parameter ratio, in case of decimator.
	fract32 * coeff;        ///< pointer to the coefficient array.
	fract32 * delayline;    ///< pointer to the internal delay line
	fract32 * head;         ///< pointer to the current write head in the delayline.
	int ratio;             ///< Interpolation / decimation ratio. effective only for the decimator and interpolator. Set to 1 for FIR filter.
	int tapsByRatio;       ///< taps / ratio pre-calcuration.
}TFIR32;

/**
 * \brief FIR initialization routine.
 * \param filter TFIR32 struct. Filled by a set of parameter for fr32_fir etc, when returned.
 * \param taps Number of taps for filter. Must be longer than 1. Must be integer multiple of ratio, for interpolator and decimator.
 * \param coeff Coefficient array. The length have to be same or longer than taps.
 * \param delayline Internal delay line array. The length have to be same or longer than taps. Will be zero filled by this function.
 * \param ratio Interpolation / Decimation ratio. Must be 1 for the FIR filter.
 * \details
 * Set the structure "filter", for the filter processing. This initializer is shared among
 * FIR filter, interpolator and decimator.
 * This routine have to be called once before invoking fr32_fir(), fr32_interpolator(), fr32_decimator() function.
 */

void fr32_fir_init( TFIR32 *filter, int taps, fract32 * coeff, fract32 * delayline, int ratio);


/**
 * \brief FIR processing routing
 * \param filter TFIR32 struct. Have to be initialized by fr32_fir_init().
 * \param in input samples buffer
 * \param out output sample buffer.
 * \param count number of input and output samples.
 * \details
 * Finite Impulse Response filter. The variable given to filter parameter have to be initialized by fr32_fir_init() function,
 * at least and once before invoking this funciton.
 */

void fr32_fir(TFIR32 *filter, fract32 in[], fract32 out[], int count );

/**
 * \brief Interpolator routing
 * \param filter TFIR32 struct. Have to be initialized by fr32_fir_init().
 * \param in input samples buffer
 * \param out output sample buffer.
 * \param count number of input samples. The number of output data in out array will be count * ratio.
 * \details
 * Interpolator. The variable given to filter parameter have to be initialized by fr32_fir_init() function,
 * at least and once before invoking this function.
 * The number of output samples are count * ratio. Where, ratio is defined by fr32_fir_init() function.
 */

void fr32_interpolator(TFIR32 *filter, fract32 in[], fract32 out[], int count );

/**
 * \brief Decimator routing
 * \param filter TFIR32 struct. Have to be initialized by fr32_fir_init().
 * \param in input samples buffer
 * \param out output sample buffer.
 * \param count number of input samples. The number of output data in out array will be count / ratio. Where ratio is specified by fr32_fir_init().
 * \details
 * Decimator. The variable given to filter parameter have to be initialized by fr32_fir_init() function, at least and once before invoking this funciton.
 * The number of output samples are count / ratio. Where, ratio is defined by fr32_fir_init() function.
 */

void fr32_decimator(TFIR32 *filter, fract32 in[], fract32 out[], int count );


#endif /* FR32_FIR_H_ */

/*@} */

